from naas_drivers.driver import InDriver, OutDriver
import pandas as pd
import requests
import os


class TKCRUD:
    def __init__(self, base_url, subdomain, auth):
        self.req_headers = {
            "X-Auth-API-Key": auth,
            "X-Auth-Subdomain": subdomain,
            "Accept": "application/json",
            "Content-Type": "application/json",
        }
        self.base_url = base_url
        self.model_name = self.base_url.split("/")[-1]

    def __get_by_page(self, page):
        data = {"page": page}
        req = requests.get(
            url=f"{self.base_url}/",
            headers=self.req_headers,
            json=data,
            allow_redirects=False,
        )
        req.raise_for_status()
        return req.json()

    def get_all(self):
        items = []
        current_page = 1
        more_page = True
        while more_page:
            data = self.__get_by_page(current_page)
            items.extend(data.get("items"))
            total_pages = data.get("meta").get("pagination").get("total_pages") or 0
            if current_page == total_pages:
                more_page = False
            current_page += 1
        df = pd.DataFrame.from_records(items)
        return df

    def get(self, uid):
        if not uid:
            raise ValueError("Uid can not be None.")

        try:
            req = requests.get(
                url=f"{self.base_url}/{uid}",
                headers=self.req_headers,
                allow_redirects=False,
            )
            req.raise_for_status()
            return req.json()
        except requests.HTTPError as err:
            err_code = err.response.status_code
            if err_code == 404:
                print(f"Uid '{uid}' not found.")
            else:
                print(err.response.json())

    def update(self, data):
        uid = data["id"]
        if not uid:
            raise ValueError("Uid can not be None.")
        try:
            req = requests.put(
                url=f"{self.base_url}/{uid}",
                headers=self.req_headers,
                json=data,
                allow_redirects=False,
            )
            req.raise_for_status()
            print(f"'{self.model_name}' '{uid}' updated.")
        except requests.HTTPError as err:
            err_code = err.response.status_code
            if err_code == 404:
                print(f"Uid '{uid}' not found.")
            else:
                print(err.response.json())

    def send(self, data):
        if len(data) == 0:
            raise ValueError("Data is empty")

        try:
            req = requests.post(
                url=f"{self.base_url}/",
                headers=self.req_headers,
                json=data,
                allow_redirects=False,
            )
            req.raise_for_status()
            return req.json()
        except requests.HTTPError as err:
            err_code = err.response.status_code
            if err_code == 422:
                print("User already exists.")
            else:
                print(err.response.json())

    def delete(self, uid):
        try:
            req = requests.delete(
                url=f"{self.base_url}/{uid}",
                headers=self.req_headers,
                allow_redirects=False,
            )
            req.raise_for_status()
            print(f"'{self.model_name}' '{uid}' delete.")
        except requests.HTTPError as err:
            err_code = err.response.status_code
            if err_code == 404:
                print(f"Uid '{uid}' not found.")
            else:
                print(err.response.json())


class User(TKCRUD):
    def create(self, email, password, first_name, last_name, company=None):
        user = {
            "email": email,
            "password": password,
            "first_name": first_name,
            "last_name": last_name,
            "company": company,
        }
        result = self.send(user)
        return result


class Courses(TKCRUD):
    def get_chapters(self, uid):
        req = requests.get(
            url=f"{self.base_url}/{uid}/chapters",
            headers=self.req_headers,
            allow_redirects=False,
        )
        req.raise_for_status()
        return req.json()


class Thinkific(InDriver, OutDriver):

    base_url = os.environ.get(
        "THINKIFIC_API_URL", "https://api.thinkific.com/api/public/v1"
    )
    token = None
    subdomain = None
    users = None
    enrollments = None

    def connect(self, api_token, subdomain):
        # Init thinkific attribute
        self.token = api_token
        self.subdomain = subdomain

        # Init end point
        self.users = User(f"{self.base_url}/users", self.subdomain, self.token)
        self.enrollments = TKCRUD(
            f"{self.base_url}/enrollments", self.subdomain, self.token
        )
        self.courses = Courses(f"{self.base_url}/courses", self.subdomain, self.token)
        self.reviews = TKCRUD(
            f"{self.base_url}/course_reviews", self.subdomain, self.token
        )

        # Set connexion to active
        self.connected = True
        return self



from naas_drivers.driver import OutDriver
import requests


class Integromat(OutDriver):
    def send(self, url, data=None):
        r = requests.post(
            url=url,
            json=data,
        )
        r.raise_for_status()
        return r.json()



import ftplib
import pysftp
import os


class Ftpbase:
    """FTP/SFTP Connector class for cashstory\nSample Credentials file :{
        "url": "",
        "protocole": "ftp or sftp",
        "port": ,
        "type": "explicit TLS",
        "auth_method": "Passif or Actif",
        "username": "",
        "password": ""
    }"""

    def __enter__(self):
        return self

    def get_file(self, path, dest_path=None):
        """Read and return data from file which is specified 'path' """
        saved_path = None
        filename = os.path.basename(path)
        if filename != path:
            saved_path = self.pwd()
            self.cwd(os.path.dirname(path))
        if self.conprotocol == "ftp":
            with open(dest_path if dest_path else filename, "wb") as f:

                def callback(data):
                    f.write(data)
                    print(f"Saved as {dest_path}")

                self.retrbinary(f"RETR {filename}", callback)
        elif self.conprotocol == "sftp":
            with self.ftp.open(path) as f:
                file_byte = open(dest_path if dest_path else path, "rb")  # file to send
                file_byte.write(f)
                file_byte.close()
                print(f"Saved as {dest_path}")

        if filename != path:
            self.cwd(saved_path)

    def send_file(self, path, dest_path=None):
        saved_path = None
        if dest_path is not None:
            saved_path = self.pwd()
            self.cwd(os.path.dirname(dest_path))
        filename = os.path.basename(path)
        ftp_command = f"STOR {filename}"
        file_byte = open(path, "rb")  # file to send
        self.storbinary(ftp_command, file_byte)  # send the file
        file_byte.close()
        if dest_path is not None:
            self.cwd(saved_path)
        print(f"Saved in {dest_path}")

    def list_directory(self, path):
        files = []
        try:
            files = self.ftp.nlst()
        except ftplib.error_perm:
            print("Error read this directory")
        return files

    def __init__(self, **credentials):
        if credentials["protocole"] == "ftp":
            ftp = (
                ftplib.FTP_TLS(credentials["url"])
                if credentials["type"] == "explicit TLS"
                else ftplib.FTP(credentials["url"])
            )
            ftp.connect(credentials["url"], credentials["port"])
            ftp.set_pasv(True) if credentials[
                "auth_method"
            ] == "Passif" else ftp.set_pasv(False)
            ftp.login(credentials["username"], credentials["password"])
            self.ftp = ftp
        elif credentials["protocole"] == "sftp":
            self.ftp = pysftp.Connection(
                host=credentials["url"],
                username=credentials["username"],
                password=credentials["password"],
                port=credentials["port"],
            )
        self.conprotocol = credentials["protocole"]

    def __exit__(self, *krgs):
        self.ftp.close()

    def help(self):
        print(f"=== {type(self).__name__} === \n")
        print(".get_file(path) => get file from ftp path\n")
        print(".send_file(path, dest_path) => send file to ftp path\n")
        print(".list_directory(path) => do ls in ftp in path\n")



import pandas as pd

basic_text = "Define it"
basic_error = "Define it, it should return a Dataframe"


class ConnectDriver:

    connected = False

    def connect(self, *args, **kwargs):
        print(basic_text, *args, **kwargs)
        self.connected = True
        return self

    def check_connect(self):
        if not self.connected:
            raise ValueError("you should call connect first")


class InDriver(ConnectDriver):
    def convert_data_to_df(self, *args, **kwargs):
        return basic_error

    def get(self, *args, **kwargs) -> pd.DataFrame:
        self.check_connect()
        print(basic_text, *args, **kwargs)
        return basic_error


class OutDriver(ConnectDriver):
    def send(self, *args, **kwargs):
        self.check_connect()
        print(basic_text, *args, **kwargs)
        return basic_error



from naas_drivers.driver import InDriver
import requests
import pandas as pd
import os


class Cityfalcon(InDriver):
    __key = None
    _url_base = os.environ.get(
        "CITYFALCON_API", "https://api.cityfalcon.com/v0.2/stories"
    )

    def connect(self, key):
        self.__key = key if key else os.environ.get("CITYFALCON_KEY", None)
        self.connected = True
        return self

    def convert_data_to_df(self, data, fields, limit) -> pd.DataFrame:
        news = []
        for element in data["stories"]:
            new_formated = {}
            for field in fields:
                if field == "title":
                    new_formated["title"] = element["title"]
                elif field == "link":
                    new_formated["link"] = element["url"]
                elif field == "description":
                    new_formated["description"] = element["description"]
                elif field == "score":
                    new_formated["score"] = element["cityfalconScore"]
                elif field == "sentiment":
                    new_formated["sentiment"] = element["sentiment"]
                elif field == "source":
                    new_formated["source"] = element["source"]["brandName"]
                elif field == "image":
                    new_formated["image"] = (
                        element["imageUrls"][0] if element["imageUrls"] else ""
                    )
                elif field == "logo":
                    new_formated["logo"] = (
                        element["source"]["imageUrls"]["large"]
                        if element["source"] and element["source"]["imageUrls"]
                        else ""
                    )
                elif field == "date":
                    new_formated["date"] = element["publishTime"]
                else:
                    raise ValueError(f"Unknow parameter {field}")
            news.append(new_formated)
        if limit and isinstance(limit, int) and limit > 0:
            news = news[:limit]
        return pd.DataFrame.from_records(news)

    def get(
        self,
        action,
        fields=[
            "image",
            "title",
            "logo",
            "link",
        ],
        country="US",
        limit=None,
        min_score=20,
        paywall=False,
        identifier_type="full_tickers",
        time_filter="d1",
        languages="en",
    ) -> pd.DataFrame:
        self.check_connect()
        url = f"{self._url_base}?access_token={self.__key}"
        url = f"{url}&identifier_type={identifier_type}&paywall={paywall}&identifiers={action}_{country}&categories=mp%2Cop"
        url = f"{url}&min_cityfalcon_score={min_score}&order_by=latest&time_filter={time_filter}&all_languages=false&languages={languages}"
        req = requests.get(url)
        dict_news = req.json()
        return self.convert_data_to_df(dict_news, fields, limit)



from naas_drivers.driver import InDriver, OutDriver
from dateutil.parser import parse
import pandas as pd
import requests
import os


class Jupyter(InDriver, OutDriver):
    base_url = os.environ.get("JUPYTERHUB_URL", "https://app.naas.ai")
    api_url = None
    token = None

    def __init__(self):
        self.api_url = f"{self.base_url}/hub/api"

    def connect(self, token):
        self.token = token
        self.connected = True
        return self

    def create_user(self, username, password):
        signup_url = f"{self.base_url}/hub/signup"
        login = {
            "username": username,
            "password": password,
        }
        headers = {"Authorization": f"token {self.token}"}
        r = requests.post(signup_url, data=login, headers=headers)
        r.raise_for_status()
        return r.json()

    def get_authorize_user(self, username):
        signup_url = f"{self.base_url}/hub/authorize/{username}"
        headers = {"Authorization": f"token {self.token}"}
        r = requests.get(signup_url, headers=headers)
        r.raise_for_status()
        return r.json()

    def change_authorize_user(self, username, is_authorized):
        signup_url = f"{self.base_url}/hub/authorize/{username}"
        headers = {"Authorization": f"token {self.token}"}
        data = {"is_authorized": is_authorized}
        r = requests.put(signup_url, json=data, headers=headers)
        r.raise_for_status()
        return r.json()

    def change_password_user(self, username, password):
        signup_url = f"{self.base_url}/hub/change-password"
        login = {
            "username": username,
            "password": password,
        }
        headers = {"Authorization": f"token {self.token}"}
        r = requests.put(signup_url, data=login, headers=headers)
        r.raise_for_status()
        return r.json()

    def list_users(self):
        signup_url = f"{self.base_url}/hub/signup"
        headers = {"Authorization": f"token {self.token}"}
        r = requests.get(signup_url, headers=headers)
        r.raise_for_status()
        df = pd.DataFrame.from_records(r.json().get("data"))
        return df

    def delete_user(self, username):
        signup_url = f"{self.base_url}/hub/signup"
        login = {
            "username": username,
        }
        headers = {"Authorization": f"token {self.token}"}
        r = requests.delete(signup_url, data=login, headers=headers)
        r.raise_for_status()
        return r.json()

    def get_me(self):
        self.connect(os.environ.get("JUPYTERHUB_API_TOKEN"))
        return self.get_user(os.environ.get("JUPYTERHUB_USER"))

    def get_me_uptime(self):
        self.connect(os.environ.get("JUPYTERHUB_API_TOKEN"))
        me = self.get_user(os.environ.get("JUPYTERHUB_USER"))
        return self.get_server_uptime(me)

    def restart_me(self):
        self.connect(os.environ.get("JUPYTERHUB_API_TOKEN"))
        username = os.environ.get("JUPYTERHUB_USER")
        return self.stop_user(username)

    def get_users(self):
        self.check_connect()
        r = requests.get(
            f"{self.api_url}/users",
            headers={
                "Authorization": f"token {self.token}",
            },
        )

        r.raise_for_status()
        return r.json()

    def get_user(self, username):
        self.check_connect()
        r = requests.get(
            f"{self.api_url}/users/{username}",
            headers={
                "Authorization": f"token {self.token}",
            },
        )

        r.raise_for_status()
        return r.json()

    def is_user_active(self, user):
        self.check_connect()
        servers = user.get("servers")
        keys = servers.keys()
        if len(keys) > 0:
            return True
        else:
            return False

    def get_server_uptime(self, user):
        self.check_connect()
        servers = user.get("servers")
        keys = servers.keys()
        all_duration = None
        for key in keys:
            server = servers[key]
            then = parse(server.get("started"))
            now = parse(server.get("last_activity"))
            duration = now - then
            if all_duration is None:
                all_duration = duration
            else:
                all_duration += duration
        return all_duration

    def stop_user(self, username):
        self.check_connect()
        r = requests.delete(
            f"{self.api_url}/users/{username}/server",
            headers={
                "Authorization": f"token {self.token}",
            },
        )
        r.raise_for_status()
        return r

    def start_user(self, username):
        self.check_connect()
        r = requests.post(
            f"{self.api_url}/users/{username}/server",
            headers={
                "Authorization": f"token {self.token}",
            },
        )
        r.raise_for_status()
        return r

    def restart_user(self, username):
        self.check_connect()
        user = self.get_user(username)
        if user and self.is_user_active(user):
            self.stop_user(username)
            self.start_user(username)



from naas_drivers.driver import InDriver
import pandas as pd
from geopy.extra.rate_limiter import RateLimiter


class Geolocator(InDriver):
    _key = None
    """GeoLocator allows users to fetch latitude and longitude for
    a list of address in dataframe column
    Example usage:
        naas_drivers.geolocator.connect(api_key="your_api_key", mode="google") #to set the service to google maps
         OR
        naas_drivers.geolocator.connect(api_key="your_app_name", mode = "osm") #to set the service to open street maps
        df = naas_drivers.geolocator.connect.get(df, column="address")
    """

    def __init__(self):
        self.client = None

    def connect(
        self,
        api_key,
        mode="google",
        domain: str = "maps.googleapis.com",
        min_delay_seconds: int = 1,
        **kwargs,
    ):
        if min_delay_seconds < 1:
            raise ValueError("Minimum 1 second delay is required")
        if mode == "google":
            from geopy.geocoders import GoogleV3

            client = GoogleV3(api_key=api_key, domain=domain, **kwargs)
            self.client = RateLimiter(
                client.geocode, min_delay_seconds=min_delay_seconds
            )
        elif mode == "som":
            from geopy.geocoders import Nominatim

            client = Nominatim(user_agent=api_key, **kwargs)
            self.client = RateLimiter(
                client.geocode, min_delay_seconds=min_delay_seconds
            )
        else:
            raise ValueError("mode should be osm or google")
        self.connected = True
        return self

    def check_connect(self):
        if not self.client:
            raise ValueError("Please set the map service using connect method")

    def get(self, df: pd.DataFrame, column: str, limit=50) -> pd.DataFrame:
        self.check_connect()
        if df.shape[0] > limit:
            raise ValueError(
                f"Dataset number of rows is more than the set limit of {limit}"
            )
        df["_location"] = df[column].apply(self.client)
        df["LATITUDE"] = df["_location"].apply(
            lambda loc: loc.latitude if loc else None
        )
        df["LONGITUDE"] = df["_location"].apply(
            lambda loc: loc.longitude if loc else None
        )
        df = df.drop(columns=["_location"])
        return df



from naas_drivers.driver import InDriver, OutDriver
from urllib.parse import urljoin
import pandas as pd
import requests
import os


BIG_NUM_TO_GETALL = 1000000


class Gsheet(InDriver, OutDriver):
    sheets_api = None
    spreadsheet_id = None
    sheet_name = None

    def __init__(self):
        self.sheets_api = os.getenv("GSHEETS_API")

    def connect(
        self,
        spreadsheet_id: str,
        api_url: str = None,
    ):
        self.spreadsheet_id = spreadsheet_id
        self.sheets_api = api_url if api_url else os.getenv("GSHEETS_API")
        self.connected = True
        return self

    def get(
        self,
        sheet_name: str,
        items_per_page: int = BIG_NUM_TO_GETALL,
    ) -> pd.DataFrame:
        self.check_connect()
        resp = requests.get(
            urljoin(self.sheets_api, f"{self.spreadsheet_id}/{sheet_name}"),
            params={"perPage": items_per_page},
        )
        resp.raise_for_status()
        data = resp.json()
        df = pd.DataFrame(data=data["data"], columns=data["columns"])
        return df

    def send(self, data, sheet_name: str):
        self.check_connect()
        resp = requests.post(
            urljoin(self.sheets_api, f"{self.spreadsheet_id}/{sheet_name}"),
            data=data,
        )
        resp.raise_for_status()
        return resp.json()
